package models

import (
	"encoding/json"
	"net/http"
	"strconv"
	"strings"
	"time"

	"forge.grandlyon.com/systemes-dinformation/project-template/sdk-go/internal/auth"
)

// HandleOperations handle API calls on Operations
func (d *DataHandler) HandleOperations(w http.ResponseWriter, r *http.Request) {
	id, _ := strconv.Atoi(strings.TrimPrefix(r.URL.Path, "/api/Operations/"))
	switch method := r.Method; method {
	case "GET":
		switch auth.GetLoggedUserTechnical(w, r).Role {
		case "ADMIN", "BANKER", "CLIENT":
			d.getOperationClient(w, r, id)
		default:
			http.Error(w, ErrorRoleOfLoggedUser, http.StatusInternalServerError)
		}
	case "POST":
		switch auth.GetLoggedUserTechnical(w, r).Role {
		case "ADMIN", "BANKER", "CLIENT":
			d.postOperationClient(w, r, id)
		default:
			http.Error(w, ErrorRoleOfLoggedUser, http.StatusInternalServerError)
		}

	case "DELETE":
		switch auth.GetLoggedUserTechnical(w, r).Role {
		case "ADMIN", "BANKER":
			d.deleteOperationBanker(w, r, id)
		case "CLIENT":
			http.Error(w, ErrorNotAuthorizeMethodOnRessource, http.StatusMethodNotAllowed)
		default:
			http.Error(w, ErrorRoleOfLoggedUser, http.StatusInternalServerError)
		}
	default:
		http.Error(w, "method not allowed", http.StatusMethodNotAllowed)
	}
}

func (d *DataHandler) getOperationClient(w http.ResponseWriter, r *http.Request, id int) {
	if id != 0 {
		var o Operation
		if err := d.db.First(&o, id).Error; err != nil {
			http.Error(w, ErrorIDDoesNotExist, http.StatusNotFound)
			return
		}
		json.NewEncoder(w).Encode(o)
	} else {
		var o []Operation
		d.db.Find(&o)
		json.NewEncoder(w).Encode(o)
	}
}

func (d *DataHandler) postOperationClient(w http.ResponseWriter, r *http.Request, id int) {
	var o Operation
	err := json.NewDecoder(r.Body).Decode(&o)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
	}

	var debtor BankAccount
	var creditor BankAccount
	if err := d.db.Where(reqID, o.Debtor).First(&debtor).Error; err != nil {
		http.Error(w, "Can not find debtor account", http.StatusInternalServerError)
		return
	}
	if err := d.db.First(&creditor, o.Creditor).Error; err != nil {
		http.Error(w, "Can not find creditor account", http.StatusInternalServerError)
		return
	}
	if (debtor.Amount + o.Amount) <= debtor.BankOverdraft {
		http.Error(w, "Not enough money", http.StatusExpectationFailed)
		return
	}
	// Update BankAccounts
	debtor.Amount += o.Amount
	creditor.Amount -= o.Amount
	d.db.Save(&debtor)
	d.db.Save(&creditor)

	now := time.Now()
	o.Date = now
	d.db.Create(&o)

	// Add the operation to creditor
	op := Operation{
		Debtor:   o.Creditor,
		Amount:   -o.Amount,
		Date:     now,
		Creditor: o.Debtor,
	}
	d.db.Create(&op)
}

func (d *DataHandler) deleteOperationBanker(w http.ResponseWriter, r *http.Request, id int) {
	if id != 0 {
		var o Operation
		if err := d.db.First(&o, id).Error; err != nil {
			http.Error(w, ErrorIDIsMissing, http.StatusNotFound)
			return
		}
		// update BankAccounts
		var debtor BankAccount
		var creditor BankAccount
		if err := d.db.First(&debtor, o.Debtor).Error; err == nil {
			if err := d.db.First(&creditor, o.Creditor).Error; err == nil {
				// Update BankAccounts
				debtor.Amount -= o.Amount
				creditor.Amount += o.Amount
				d.db.Save(&debtor)
				d.db.Save(&creditor)

				// Get the operation of the creditor
				var op Operation
				if err := d.db.First(&op, id+1).Error; err != nil {
					http.Error(w, ErrorIDIsMissing, http.StatusNotFound)
					return
				}

				// Delete the operations
				d.db.Delete(&o)
				d.db.Delete(&op)
			}
		}
	} else {
		http.Error(w, ErrorIDIsMissing, http.StatusNotFound)
	}
}
