package rootmux

import (
	"encoding/json"
	"testing"

	"forge.grandlyon.com/systemes-dinformation/project-template/sdk-go/internal/auth"
)

/**
Banker TESTS (those tests are to check the bankers rights)
**/
func BankerTests(t *testing.T) {
	// Create the tester
	ts, do, _ := createTester(t)
	defer ts.Close() // Close the tester
	tests := func() {
		// Get the XSRF Token
		response := do("GET", "/api/common/WhoAmI", noH, "", 200, "")
		token := auth.TokenData{}
		json.Unmarshal([]byte(response), &token)
		xsrfHeader := map[string]string{"XSRF-TOKEN": token.XSRFToken}

		// Create a banker should fail with 405
		do("POST", "/api/UserBankers", xsrfHeader, `{"UserID":8,"Name":"Banker 2"}`, 405, `You're not authorize to execute this method on this ressource.`)
		// Get the banker connected
		do("GET", "/api/UserBankers/1", xsrfHeader, "", 200, `{"ID":1,"UserID":3,"Name":"Banker","UserClients":[{"ID":1,"UserID":1,"Name":"Dupond","UserBankerID":1,"BankAccounts":null}]}`)
		// Try to get an other banker
		do("GET", "/api/UserBankers/2", xsrfHeader, "", 403, `You can not access this ressource`)
		// Try to gel all bankers
		do("GET", "/api/UserBankers/", xsrfHeader, "", 403, `You can not access this ressource`)
		// Try to delete an other banker should fail with 405
		do("DELETE", "/api/UserBankers/2", xsrfHeader, ``, 405, `You're not authorize to execute this method on this ressource.`)

		// Try to create a client
		do("POST", "/api/UserClients", xsrfHeader, `{"UserID":7,"Name":"Dupond","UserBankerID":1}`, 200, "")
		// Try to get one of the banker's client
		do("GET", "/api/UserClients/1", xsrfHeader, "", 200, `{"ID":1,"UserID":1,"Name":"Dupond","UserBankerID":1,"BankAccounts":[{"ID":1,"Number":"01-01","UserClientID":1,"Type":"checking-account","Amount":458,"BankOverdraft":-100,"Operations":null}]}`)
		// Try to get another banker's client should fail with 405
		do("GET", "/api/UserClients/2", xsrfHeader, "", 403, `You can not access this ressource`)
		// Try to get all the clients of the banker
		do("GET", "/api/UserClients", xsrfHeader, "", 200, `[{"ID":1,"UserID":1,"Name":"Dupond","UserBankerID":1,"BankAccounts":[{"ID":1,"Number":"01-01","UserClientID":1,"Type":"checking-account","Amount":458,"BankOverdraft":-100,"Operations":null}]},{"ID":3,"UserID":7,"Name":"Dupond","UserBankerID":1,"BankAccounts":[]}]`)
		// Try to delete a banker client
		do("DELETE", "/api/UserClients/3", xsrfHeader, ``, 200, ``)
		// Try to delete the bakery client should fail with 405
		do("DELETE", "/api/UserClients/2", xsrfHeader, ``, 405, `You're not authorize to execute this method on this ressource.`)

		// Add an other bank account to client
		do("POST", "/api/BankAccounts", xsrfHeader, `{"Number":"01-02","UserClientID":1,"Type":"saving-account","Amount":1287,"BankOverdraft":0}`, 200, ``)
		// Add a bank account to another banker's client should fail with 405
		do("POST", "/api/BankAccounts", xsrfHeader, `{"Number":"02-02","UserClientID":2,"Type":"saving-account","Amount":3978,"BankOverdraft":0}`, 403, `You can not access this ressource`)
		// Get account where id=1
		do("GET", "/api/BankAccounts/1", xsrfHeader, "", 200, `{"ID":1,"Number":"01-01","UserClientID":1,"Type":"checking-account","Amount":458,"BankOverdraft":-100,"Operations":[]}`)
		// Get account where id=2 should fail as it is another banker's client's account
		do("GET", "/api/BankAccounts/2", xsrfHeader, "", 403, `You can not access this ressource`)
		// Get all Bank account should return only the bank accounts of the bankers'client
		do("GET", "/api/BankAccounts/", xsrfHeader, "", 200, `[{"ID":1,"Number":"01-01","UserClientID":1,"Type":"checking-account","Amount":458,"BankOverdraft":-100,"Operations":[]},{"ID":3,"Number":"01-02","UserClientID":1,"Type":"saving-account","Amount":1287,"BankOverdraft":0,"Operations":[]}]`)
		// Try to delete the saving account of Dupond
		do("DELETE", "/api/BankAccounts/3", xsrfHeader, ``, 200, "")
		// Try to delete the saving account of Bakery should fail with 405
		do("DELETE", "/api/BankAccounts/2", xsrfHeader, ``, 403, "You can not access this ressource")

		// Add operation between client and Bakery
		do("POST", "/api/Operations", xsrfHeader, `{"Debtor":1,"Amount":-100,"Creditor":2}`, 200, "")
		// Get operation where id=1
		do("GET", "/api/Operations/1", xsrfHeader, "", 200, `{"ID":1,"Debtor":1,"Amount":-100`)
		// Try to delete the first operation
		do("DELETE", "/api/Operations/1", xsrfHeader, ``, 200, "")
	}
	// Do a in memory login with an known admin
	do("POST", "/Login", noH, `{"login": "banker","password": "password"}`, 200, "")
	tests()
	// Try to logout (must pass)
	do("GET", "/Logout", noH, "", 200, "Logout OK")
}
