package rootmux

import (
	"net/http"
	"testing"
)

/**
UNLOGGED USER TESTS (this tests are to check that the security protections works)
**/

const errorExtractingToken = `error extracting token`

func UnLoggedUserTests(t *testing.T) {
	// Create the tester
	ts, do, _ := createTester(t)
	defer ts.Close() // Close the tester
	// Try to access the main page (must pass)
	do("GET", "/", noH, "", 200, "<!DOCTYPE html>")
	// Try to get the user informations (must fail)
	do("GET", "/api/common/WhoAmI", noH, "", 401, "error extracting token")
	// Do a in memory login with an unknown user
	do("POST", "/Login", noH, `{"login": "unknownuser","password": "password"}`, http.StatusForbidden, `user not found`)
	// Do a in memory login with a known user but bad password
	do("POST", "/Login", noH, `{"login": "admin","password": "badpassword"}`, http.StatusForbidden, `user not found`)

	// Without authent API calls must fails with 401 error extracting token
	// Try to get the first client should fail
	do("GET", "/api/UserClients/1", noH, "", 401, errorExtractingToken)
	// Try to get all clients should fail
	do("GET", "/api/UserClients", noH, "", 401, errorExtractingToken)
	// Try to get the first operation should fail
	do("GET", "/api/UserBankers/1", noH, "", 401, errorExtractingToken)
	// Try to get all clients should fail
	do("GET", "/api/UserBankers", noH, "", 401, errorExtractingToken)
	// Try to get the first operation should fail
	do("GET", "/api/Operations/1", noH, "", 401, errorExtractingToken)
	// Try to get all operations should fail
	do("GET", "/api/Operations", noH, "", 401, errorExtractingToken)
	// Try to get the first BankAccount should fail
	do("GET", "/api/BankAccounts/1", noH, "", 401, errorExtractingToken)
	// Try to get all BankAccounts should fail
	do("GET", "/api/BankAccounts", noH, "", 401, errorExtractingToken)

	// Unlogged user should not be able to create a Client
	do("POST", "/api/UserClients", noH, `{"Name":"Dupond"}`, 401, errorExtractingToken)
	// Unlogged user should not be able to create a Banker
	do("POST", "/api/UserBankers", noH, `{"Name":"Dupond"}`, 401, errorExtractingToken)
	// Unlogged user should not be able to create a Bank Account
	do("POST", "/api/BankAccounts", noH, `{"Number":"01-01","UserClientID":1,"Type":"checking-account","Amount":458,"BankOverdraft":-100}`, 401, errorExtractingToken)
	// Unlogged user should not be able to create an Operation
	do("POST", "/api/Operations", noH, `{"Debtor":1,"Amount":-100,"Creditor":3}`, 401, errorExtractingToken)

	// Unlogged user should not be able to delete an Operation
	do("DELETE", "/api/Operations/1", noH, ``, 401, errorExtractingToken)
	// Unlogged user should not be able to delete a BankAccount
	do("DELETE", "/api/BankAccounts/2", noH, ``, 401, errorExtractingToken)
	// Unlogged user should not be able to delete a Client
	do("DELETE", "/api/UserClients/2", noH, ``, 401, errorExtractingToken)
	// Unlogged user should not be able to delete a Banker
	do("DELETE", "/api/UserBankers/2", noH, ``, 401, errorExtractingToken)
}
