// Imports
import * as Auth from "/services/auth/auth.js";
import * as Messages from "/services/messages/messages.js";
import * as Operations from "/components/bankerPage/operations.js";

// DOM elements

// local variables
let current_user;

export async function mount(where, parent) {
  const bankAccount = new BankAccount(parent);
  await bankAccount.mount(where);
  return bankAccount;
}

class BankAccount {
  constructor(parent) {
    this.parent = parent;
  }

  async mount(where) {
    current_user = await Auth.GetUser();

    document.getElementById(where).innerHTML = /* HTML */ `
      <div class="container">
        <div class="columns">
          <div class="column is-one-third">
            <h5 class="title is-5">Informations du compte bancaire <em id="account-number"></em></h5>
            <div class="container">
              <div class="field" id="users-modal-role-container">
                <label>Type de compte</label><br />
                <div class="control select">
                  <select name="type" id="account-type" ${current_user.role == "CLIENT" ? `disabled` : ``}>
                    <option value="checking-account">Compte courant</option>
                    <option value="saving-account">Compte épargne</option>
                  </select>
                </div>
              </div>
              <div class="field">
                <label>Découvert autorisé</label>
                <div class="control">
                  <input class="input" type="number" id="account-overdraft" ${current_user.role == "CLIENT" ? `disabled` : ``} />
                </div>
              </div>
              ${current_user.role != "CLIENT"
                ? /* HTML */ ` <button id="account-save" class="button is-success">Sauvegarder</button>
                    <button id="account-cancel" class="button">Annuler</button>`
                : ``}
            </div>
          </div>
          <div class="column">
            <h5 class="title is-5">Opérations du compte bancaire</h5>
            <div id="operations"></div>
          </div>
        </div>
      </div>
    `;
  }

  async handleDom() {
    document.getElementById("account-number").innerHTML = this.bankAccount.Number;
    document.getElementById("account-type").value = this.bankAccount.Type;
    document.getElementById("account-overdraft").value = this.bankAccount.BankOverdraft;
    let operationHandler = await Operations.mount("operations", this);
    await operationHandler.displayOperations(this.bankAccount);

    if (current_user.role != "CLIENT") {
      let bankAccount = this;
      document.getElementById(`account-cancel`).addEventListener("click", function () {
        bankAccount.handleDom();
      });
      document.getElementById(`account-save`).addEventListener("click", function () {
        bankAccount.bankAccount.BankOverdraft = parseInt(document.getElementById("account-overdraft").value);
        bankAccount.bankAccount.Type = document.getElementById("account-type").value;
        bankAccount.postBankAccount();
      });
    }
  }

  editBankAccount(bankAccount) {
    this.bankAccount = bankAccount;
    this.isNew = false;
    this.handleDom();
  }

  newBankAccount(client) {
    this.bankAccount = {};
    this.bankAccount.ID = 0;
    this.bankAccount.Number = "0" + client.ID + "-0" + (client.BankAccounts.length + 1);
    this.bankAccount.UserClientID = client.ID;
    this.bankAccount.Type = document.getElementById("account-type").value;
    this.bankAccount.Amount = 0;
    this.bankAccount.BankOverdraft = 0;
    this.bankAccount.Operations = [];
    this.isNew = true;
    this.handleDom();
  }

  viewBankAccount(bankAccount) {
    this.bankAccount = bankAccount;
    this.handleDom();
  }

  async deleteAccount(bankAccount) {
    try {
      const response = await fetch("/api/BankAccounts/" + bankAccount.ID, {
        method: "delete",
        headers: new Headers({
          "XSRF-Token": current_user.xsrftoken,
        }),
      });
      if (response.status !== 200) {
        throw new Error(`BankAccount could not be deleted (status ${response.status})`);
      }
      document.getElementById(`accounts-account-${bankAccount.ID}`).remove();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
  }

  async postBankAccount() {
    let method;
    if (this.isNew) method = "POST";
    else method = "PUT";

    try {
      const response = await fetch("/api/BankAccounts/" + this.bankAccount.ID, {
        method: method,
        headers: new Headers({
          "XSRF-Token": current_user.xsrftoken,
        }),
        body: JSON.stringify({
          ID: this.bankAccount.ID,
          Number: this.bankAccount.Number,
          UserClientID: this.bankAccount.UserClientID,
          Type: this.bankAccount.Type,
          Amount: this.bankAccount.Amount,
          BankOverdraft: this.bankAccount.BankOverdraft,
        }),
      });
      if (response.status !== 200) {
        throw new Error(`Banker could not be updated (status ${response.status})`);
      }
      await this.parent.updateBankAccounts();
      this.bankAccount = await response.json();
      document.getElementById(`accounts-account-edit-${this.bankAccount.ID}`).click();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
  }
}
