// Imports
import * as Messages from "/services/messages/messages.js";
import * as Auth from "/services/auth/auth.js";
import * as ClientDetails from "/components/bankerPage/clientDetails.js";

// DOM elements

// local variables
let clients;
let current_user;

export async function mount(where) {
  const bankerPage = new BankerPage();
  await bankerPage.mount(where);
  await bankerPage.displayClients();
}

class BankerPage {
  constructor() {}

  async mount(where) {
    document.getElementById(where).innerHTML = /* HTML */ `
      <div class="container">
        <h1 class="title">Page du banquier</h1>
      </div>
      <section class="section">
        <div class="container">
          <div class="columns">
            <div class="column is-one-third">
            <h5 class="title is-5">Mes clients</h5>
              <div class="table-container">
                <table class="table is-bordered is-narrow is-hoverable is-fullwidth">
                  <thead>
                    <tr class="is-selected">
                      <th>Id</th>
                      <th>Nom</th>
                    </tr>
                  </thead>
                  <tbody id="clients"></tbody>
                </table>
              </div>
            </div>
            <div class="column">
            <h5 class="title is-5">Comptes bancaires de <em id="client-title">Veuillez sélectionner un client</em></h5>
              <div class="container">
                <div id="bank-accounts"></div>
              </div>
            </div>
          </div>
        </div>
      </section>
      <section class="section>
        <div class="container" id="bank-account"></div>
      </section>
    `;
    current_user = await Auth.GetUser();
  }

  async displayClients() {
    let clients = [];
    try {
      const response = await fetch("/api/UserClients/", {
        method: "GET",
        headers: new Headers({
          "XSRF-Token": current_user.xsrftoken,
        }),
      });
      if (response.status !== 200) {
        throw new Error(`Users could not be fetched (status ${response.status})`);
      }
      clients = await response.json();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
    const markup = clients.map((client) => this.clientTemplate(client)).join("");
    document.getElementById("clients").innerHTML = markup;
    clients.map((client) => {
      document.getElementById(`clients-client-${client.ID}`).addEventListener("click", function () {
        document.getElementById("client-title").innerHTML = client.Name;
        ClientDetails.mount("bank-accounts", client);
      });
    });
  }

  clientTemplate(client) {
    return /* HTML */ `
      <tr id="clients-client-${client.ID}">
        <td>${client.ID}</td>
        <td>${client.Name}</td>
      </tr>
    `;
  }
}
