// Imports
import * as Auth from "/services/auth/auth.js";
import * as Messages from "/services/messages/messages.js";
import * as BankAccount from "/components/bankerPage/bankAccount.js";

// DOM elements

// local variables
let current_user;

export async function mount(where, client) {
  const clientDetails = new ClientDetails(client);
  await clientDetails.mount(where);
  clientDetails.displayBankAccounts();
}

class ClientDetails {
  constructor(client) {
    this.client = client;
  }

  async mount(where) {
    current_user = await Auth.GetUser();

    document.getElementById(where).innerHTML = /* HTML */ `
      <div class="table-container">
        <table class="table is-bordered is-narrow is-hoverable is-fullwidth">
          <thead>
            <tr class="is-selected">
              <th>Id</th>
              <th>Numéro</th>
              <th>Type</th>
              <th>Montant</th>
              <th>Découvert autorisé</th>
              <th>Actions</th>
            </tr>
          </thead>
          <tbody id="accounts"></tbody>
        </table>

        ${current_user.role != "CLIENT"
          ? `
        <nav class="level">
          <div class="level-left"></div>
          <!-- Right side -->
          <div class="level-right">
            <button id="bankAccount-new" class="button is-success">
              <span class="icon is-small">
                <i class="fas fa-plus"></i>
              </span>
            </button>
          </div>
        </nav>`
          : ``}
      </div>
    `;
  }

  async displayBankAccounts() {
    const markup = this.client.BankAccounts.map((bankAccount) => this.bankAccountTemplate(bankAccount)).join("");
    document.getElementById("accounts").innerHTML = markup;
    let accountHandler = await BankAccount.mount("bank-account", this);

    this.client.BankAccounts.map((bankAccount) => {
      if (current_user.role == "CLIENT") {
        document.getElementById(`accounts-account-view-${bankAccount.ID}`).addEventListener("click", function () {
          document.getElementById("account-title").innerHTML = bankAccount.Number;
          accountHandler.viewBankAccount(bankAccount);
        });
      } else {
        document.getElementById(`accounts-account-edit-${bankAccount.ID}`).addEventListener("click", function () {
          accountHandler.editBankAccount(bankAccount);
        });
        document.getElementById(`accounts-account-delete-${bankAccount.ID}`).addEventListener("click", function () {
          accountHandler.deleteAccount(bankAccount);
        });
      }
    });
    let clientDetails = this;
    document.getElementById(`bankAccount-new`).addEventListener("click", function () {
      console.log("test");
      accountHandler.newBankAccount(clientDetails.client);
    });
  }

  bankAccountTemplate(bankAccount) {
    return /* HTML */ `
      <tr id="accounts-account-${bankAccount.ID}">
        <td>${bankAccount.ID}</td>
        <td>${bankAccount.Number}</td>
        <td>${bankAccount.Type}</td>
        <td>${bankAccount.Amount}</td>
        <td>${bankAccount.BankOverdraft}</td>
        <td>
          ${current_user.role == "CLIENT"
            ? `<a id="accounts-account-view-${bankAccount.ID}" class="button is-link is-small">
        <span>Visualiser</span>
        <span class="icon is-small">
          <i class="fas fa-pen"></i>
        </span>
      </a>`
            : `<a id="accounts-account-edit-${bankAccount.ID}" class="button is-link is-small">
            <span>Modifier</span>
            <span class="icon is-small">
              <i class="fas fa-pen"></i>
            </span>
          </a>
          <a id="accounts-account-delete-${bankAccount.ID}" class="button is-danger is-small">
            <span>Supprimer</span>
            <span class="icon is-small">
              <i class="fas fa-trash"></i>
            </span>
          </a>`}
        </td>
      </tr>
    `;
  }

  async updateBankAccounts() {
    try {
      const response = await fetch("/api/UserClients/" + this.client.ID, {
        method: "GET",
        headers: new Headers({
          "XSRF-Token": current_user.xsrftoken,
        }),
      });
      if (response.status !== 200) {
        throw new Error(`Users could not be fetched (status ${response.status})`);
      }
      this.client = await response.json();
      await this.displayBankAccounts();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
  }
}
