// Imports
import * as Messages from "/services/messages/messages.js";
import * as Auth from "/services/auth/auth.js";
import * as Users from "/components/users/users.js";

// DOM elements
let name_field;
let banker_name_field;
let banker_id_field;

// local variables
let current_user;
let update_banker;
let user_id;

export async function mount(where, parent) {
  const handleBanker = new HandleBanker(parent);
  await handleBanker.mount(where);
  handleBanker.handleDom();
  return handleBanker;
}

class HandleBanker {
  constructor(parent) {
    this.parent = parent;
  }

  async mount(where) {
    document.getElementById(where).innerHTML = /* HTML */ `
      <div class="modal-background"></div>
      <div class="modal-card" id="banker-modal-card">
        <header class="modal-card-head">
          <p class="modal-card-title">Ajout/Supression d'un banquier</p>
          <button class="delete" aria-label="close" id="banker-modal-close"></button>
        </header>
        <section class="modal-card-body">
          <div class="field">
            <label>Id</label>
            <div class="control">
              <input class="input" type="number" id="banker-modal-id" disabled />
            </div>
          </div>
          <div class="field">
            <label>Nom</label>
            <div class="control">
              <input class="input" type="text" id="banker-modal-name" />
            </div>
          </div>
        </section>
        <footer class="modal-card-foot">
          <button id="banker-modal-save" class="button is-success">Sauvegarder</button>
          <button id="banker-modal-cancel" class="button">Annuler</button>
        </footer>
      </div>
    `;
    current_user = await Auth.GetUser();
  }

  handleDom() {
    name_field = document.getElementById("users-modal-name");
    banker_id_field = document.getElementById("banker-modal-id");
    banker_name_field = document.getElementById("banker-modal-name");

    document.getElementById(`banker-modal-close`).addEventListener("click", function () {
      Users.toggleModal("banker-modal", "banker-modal-card");
    });
    document.getElementById(`banker-modal-cancel`).addEventListener("click", function () {
      Users.toggleModal("banker-modal", "banker-modal-card");
    });
    let bankerHandler = this
    document.getElementById(`banker-modal-save`).addEventListener("click", function () {
      bankerHandler.postBanker();
    });
  }

  async editBanker(banker) {
    update_banker = true;
    banker_id_field.value = banker.ID;
    banker_name_field.value = banker.Name;
    Users.toggleModal("banker-modal", "banker-modal-card");
  }

  async newBanker(userID) {
    update_banker = false;
    user_id = userID;
    banker_name_field.value = name_field.value;
    Users.toggleModal("banker-modal", "banker-modal-card");
  }

  async deleteBanker(banker) {
    try {
      const response = await fetch("/api/UserBankers/" + banker.ID, {
        method: "delete",
        headers: new Headers({
          "XSRF-Token": current_user.xsrftoken,
        }),
      });
      if (response.status !== 200) {
        throw new Error(`Banker could not be deleted (status ${response.status})`);
      }
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
  }

  async postBanker() {
    let method;
    if (update_banker) method = "PUT";
    else method = "POST";

    try {
      const response = await fetch("/api/UserBankers/" + banker_id_field.value, {
        method: method,
        headers: new Headers({
          "XSRF-Token": current_user.xsrftoken,
        }),
        body: JSON.stringify({
          ID: parseInt(banker_id_field.value),
          Name: banker_name_field.value,
          UserID: parseInt(user_id),
        }),
      });
      if (response.status !== 200) {
        throw new Error(`Banker could not be updated (status ${response.status})`);
      }
      this.parent.displayUsers();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
    Users.toggleModal("banker-modal", "banker-modal-card");
  }
}
