// Imports
import * as Messages from "/services/messages/messages.js";
import * as Auth from "/services/auth/auth.js";
import * as Users from "/components/users/users.js";

// DOM elements
let name_field;
let client_id_field;
let client_name_field;
let client_bankerID_field;

// local variables
let current_user;
let bankers = [];
let update_client;
let user_id;

export async function mount(where, parent) {
  const handleClient = new HandleClient(parent);
  await handleClient.mount(where);
  handleClient.handleDom();
  return handleClient;
}

class HandleClient {
  constructor(parent) {
    this.parent = parent;
  }

  async mount(where) {
    document.getElementById(where).innerHTML = /* HTML */ `
      <div class="modal-background"></div>
      <div class="modal-card" id="client-modal-card">
        <header class="modal-card-head">
          <p class="modal-card-title">Ajout/Édition d'un client</p>
          <button class="delete" aria-label="close" id="client-modal-close"></button>
        </header>
        <section class="modal-card-body">
          <div class="field">
            <label>Id</label>
            <div class="control">
              <input class="input" type="number" id="client-modal-id" disabled />
            </div>
          </div>
          <div class="field">
            <label>Nom</label>
            <div class="control">
              <input class="input" type="text" id="client-modal-name" />
            </div>
          </div>
          <div class="field" id="users-modal-role-container">
            <label>Banquier</label><br />
            <div class="control select">
              <select name="banker" id="client-modal-banker-id"> </select>
            </div>
          </div>
        </section>
        <footer class="modal-card-foot">
          <button id="client-modal-save" class="button is-success">Sauvegarder</button>
          <button id="client-modal-cancel" class="button">Annuler</button>
        </footer>
      </div>
    `;
    current_user = await Auth.GetUser();
  }

  handleDom() {
    name_field = document.getElementById("users-modal-name");
    client_id_field = document.getElementById("client-modal-id");
    client_name_field = document.getElementById("client-modal-name");
    client_bankerID_field = document.getElementById("client-modal-banker-id");

    document.getElementById(`client-modal-close`).addEventListener("click", function () {
      Users.toggleModal("client-modal", "client-modal-card");
    });
    document.getElementById(`client-modal-cancel`).addEventListener("click", function () {
      Users.toggleModal("client-modal", "client-modal-card");
    });
    let clientHandler = this;
    document.getElementById(`client-modal-save`).addEventListener("click", function () {
      clientHandler.postClient();
    });
  }

  async editClient(client) {
    await this.refreshBankers();
    update_client = true;
    client_id_field.value = client.ID;
    client_name_field.value = client.Name;
    client_bankerID_field.value = client.UserBankerID;
    Users.toggleModal("client-modal", "client-modal-card");
  }

  async newClient(userID) {
    await this.refreshBankers();
    update_client = false;
    user_id = userID;
    console.log(user_id)
    client_name_field.value = name_field.value;
    client_bankerID_field.value = 0;
    Users.toggleModal("client-modal", "client-modal-card");
  }

  async deleteClient(client) {
    try {
      const response = await fetch("/api/UserClients/" + client.ID, {
        method: "delete",
        headers: new Headers({
          "XSRF-Token": current_user.xsrftoken,
        }),
      });
      if (response.status !== 200) {
        throw new Error(`Client could not be deleted (status ${response.status})`);
      }
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
  }

  async refreshBankers() {
    const response = await fetch("/api/UserBankers/", {
      method: "GET",
      headers: new Headers({
        "XSRF-Token": current_user.xsrftoken,
      }),
    });
    if (response.status !== 200) {
      throw new Error(`Users could not be updated (status ${response.status})`);
    }
    bankers = await response.json();

    for (let i = client_bankerID_field.options.length - 1; i >= 0; i--) {
      client_bankerID_field.remove(i);
    }

    let el = document.createElement("option");
    el.textContent = "Veuillez sélectionner un banquier";
    el.value = 0;
    client_bankerID_field.appendChild(el);
    bankers.forEach((banker) => {
      el = document.createElement("option");
      el.textContent = banker.Name;
      el.value = banker.ID;
      client_bankerID_field.appendChild(el);
    });
  }

  async postClient() {
    let method;
    if (update_client) method = "PUT";
    else method = "POST";

    try {
      const response = await fetch("/api/UserClients/" + client_id_field.value, {
        method: method,
        headers: new Headers({
          "XSRF-Token": current_user.xsrftoken,
        }),
        body: JSON.stringify({
          ID: parseInt(client_id_field.value),
          UserID: parseInt(user_id),
          Name: client_name_field.value,
          UserBankerID: parseInt(client_bankerID_field.value),
        }),
      });
      if (response.status !== 200) {
        throw new Error(`Client could not be updated (status ${response.status})`);
      }
      this.parent.displayUsers();
    } catch (e) {
      Messages.Show("is-warning", e.message);
      console.error(e);
    }
    Users.toggleModal("client-modal", "client-modal-card");
  }
}
